#
# Autogenerated by Thrift Compiler (0.11.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec

import sys

from thrift.transport import TTransport
all_structs = []


class MessageTypes(object):
    ProtocolCompatibleRequest = 1
    ProtocolCompatibleResponse = 2
    ProtocolIncompatibleResponse = 4
    ErrorResponse = 6
    WarningResponse = 8
    ShutdownNotification = 10
    DisconnectNotification = 12
    DisconnectRequest = 13
    ClearTextTunnelRequest = 21
    TunnelSupportedResponse = 22
    AuthenticationRequest = 23
    AuthenticatePasswordRequest = 24
    AuthenticatePasswordResponse = 25
    AuthenticationOkResponse = 26
    SessionChangeRequest = 27
    SessionChangeResponse = 28
    QueryExecuteRequest = 29
    QueryReceiptResponse = 30
    RowMetadataResponse = 32
    RowDataResponse = 34
    StatementResponse = 36
    QueryExecuteResponse = 38
    QueryCancellationRequest = 39
    QueryCancellationResponse = 40
    QueryProgressRequest = 41
    QueryProgressResponse = 42
    SessionMonitoringRequest = 43
    SessionProtocolVersionResponse = 44
    SessionDataResponse = 46
    SessionDroppedMessagesResponse = 48
    SessionMonitoringResponse = 50
    ParseStatementRequest = 51
    ParseStatementResponse = 52
    StatementBindAndExecuteRequest = 53
    ClosePreparedStatementRequest = 55
    ClosePreparedStatementResponse = 56

    _VALUES_TO_NAMES = {
        1: "ProtocolCompatibleRequest",
        2: "ProtocolCompatibleResponse",
        4: "ProtocolIncompatibleResponse",
        6: "ErrorResponse",
        8: "WarningResponse",
        10: "ShutdownNotification",
        12: "DisconnectNotification",
        13: "DisconnectRequest",
        21: "ClearTextTunnelRequest",
        22: "TunnelSupportedResponse",
        23: "AuthenticationRequest",
        24: "AuthenticatePasswordRequest",
        25: "AuthenticatePasswordResponse",
        26: "AuthenticationOkResponse",
        27: "SessionChangeRequest",
        28: "SessionChangeResponse",
        29: "QueryExecuteRequest",
        30: "QueryReceiptResponse",
        32: "RowMetadataResponse",
        34: "RowDataResponse",
        36: "StatementResponse",
        38: "QueryExecuteResponse",
        39: "QueryCancellationRequest",
        40: "QueryCancellationResponse",
        41: "QueryProgressRequest",
        42: "QueryProgressResponse",
        43: "SessionMonitoringRequest",
        44: "SessionProtocolVersionResponse",
        46: "SessionDataResponse",
        48: "SessionDroppedMessagesResponse",
        50: "SessionMonitoringResponse",
        51: "ParseStatementRequest",
        52: "ParseStatementResponse",
        53: "StatementBindAndExecuteRequest",
        55: "ClosePreparedStatementRequest",
        56: "ClosePreparedStatementResponse",
    }

    _NAMES_TO_VALUES = {
        "ProtocolCompatibleRequest": 1,
        "ProtocolCompatibleResponse": 2,
        "ProtocolIncompatibleResponse": 4,
        "ErrorResponse": 6,
        "WarningResponse": 8,
        "ShutdownNotification": 10,
        "DisconnectNotification": 12,
        "DisconnectRequest": 13,
        "ClearTextTunnelRequest": 21,
        "TunnelSupportedResponse": 22,
        "AuthenticationRequest": 23,
        "AuthenticatePasswordRequest": 24,
        "AuthenticatePasswordResponse": 25,
        "AuthenticationOkResponse": 26,
        "SessionChangeRequest": 27,
        "SessionChangeResponse": 28,
        "QueryExecuteRequest": 29,
        "QueryReceiptResponse": 30,
        "RowMetadataResponse": 32,
        "RowDataResponse": 34,
        "StatementResponse": 36,
        "QueryExecuteResponse": 38,
        "QueryCancellationRequest": 39,
        "QueryCancellationResponse": 40,
        "QueryProgressRequest": 41,
        "QueryProgressResponse": 42,
        "SessionMonitoringRequest": 43,
        "SessionProtocolVersionResponse": 44,
        "SessionDataResponse": 46,
        "SessionDroppedMessagesResponse": 48,
        "SessionMonitoringResponse": 50,
        "ParseStatementRequest": 51,
        "ParseStatementResponse": 52,
        "StatementBindAndExecuteRequest": 53,
        "ClosePreparedStatementRequest": 55,
        "ClosePreparedStatementResponse": 56,
    }
fix_spec(all_structs)
del all_structs
